# Launch Dispersion Study with STK/Astrogator
# Analytical Graphics, Inc.
# Written by John Carrico and Mike Shepherd, March, 2000
#
# Version history
# - 10 Apr 2000:  jcarrico; Formated output & fixed RAAN problem
#
# ---

print "\n\t\tLaunch Dispersion Study \n";

#------------------------------------------------------
#------------------------------------------------------
#------------------------------------------------------
#--- User Input

# - Number of trials to run (400 to 1000 for a real case, 5 to 10 for a quick demo):

#$numPoints = 1000;

print "\nEnter the number of trial points (7 to demo, 400 to 1000 for real): ";

$numPoints = <STDIN>;

# - Launch dispersions at insertion into LEO:  1 sigma values (km, km/sec, degrees)

$sigAlt = 5.0;
$sigVel = 0.001;
$sigLat = 0.1;
$sigLon = 0.1;

# - Name of the satellite in the scenario:

$sat = "*/Satellite/Sat1";

# - Name of Launch segment in the MCS:

$launchObj = "WallopsLaunch";

# - Name of the segment to evaluate results:

$evalObj   = "Prop10Mins";
$mnvr1     = "RaiseApogee.SegmentList.Mnvr1";
$mnvr2     = "RaisePerigee.SegmentList.Mnvr2";

# - Start and Stop time:

$start_time = "\"15 Dec 2001 00:00:00.00\"";
$stop_time  = "\"25 Jan 2002 00:00:00.00\"";

#------------------------------------------------------
#------------------------------------------------------
#------------------------------------------------------

#--- Connect to STK and initialize

use Stk;

Connect_to_STK(5001, localhost, STKSocket1);
print "\nAckOn";
AckOn (STKSocket1);
print "\tConControl";
Send_Command_to_STK (STKSocket1, "ConControl / VerboseOn");
print "\tConnected!!!\n";

#--- Initialize random number:

srand;

#--- Set format for numerical output

$form = "%-10.4f";

#--- Get initial control parameters

$initAltitude      = GetMCSSegmentValue(STKSocket1, $sat, $launchObj, "Burnout.Geodetic.Altitude");
$initLatitude      = GetMCSSegmentValue(STKSocket1, $sat, $launchObj, "Burnout.Geodetic.Latitude");
$initLongitude     = GetMCSSegmentValue(STKSocket1, $sat, $launchObj, "Burnout.Geodetic.Longitude");
$initFixedVelocity = GetMCSSegmentValue(STKSocket1, $sat, $launchObj, "Burnout.FixedVelocity");


print  "\nInitial Parameters:";
printf "\n        Altitude: $form",$initAltitude;
printf "\n        Latitude: $form",$initLatitude;
printf "\n       Longitude: $form",$initLongitude;
printf "\n   FixedVelocity: $form",$initFixedVelocity;
print  "\n";

#--- Run Monte Carlo

print "\nRunning $numPoints trials:";

for ($k=1; $k <= $numPoints; $k++)
{
	# --- Set perturbed control parameters
	
    $Altitude     = $initAltitude      + rand (2.0*$sigAlt) - $sigAlt;
    $Latitude     = $initLatitude      + rand (2.0*$sigLat) - $sigLat;
    $Longitude    = $initLongitude     + rand (2.0*$sigLon) - $sigLon;
    $FixedVelocity= $initFixedVelocity + rand (2.0*$sigVel) - $sigVel;
	
	print  "\n>$k: ";
    printf " Alt: $form; Lat: $form; Lon: $form; Vel: $form",$Altitude,$Latitude,$Longitude,$FixedVelocity;
	
    SetMCSSegmentValue(STKSocket1, $sat, $launchObj,"Burnout.Geodetic.Altitude",  "$Altitude km");
    SetMCSSegmentValue(STKSocket1, $sat, $launchObj,"Burnout.Geodetic.Latitude",  "$Latitude deg");
    SetMCSSegmentValue(STKSocket1, $sat, $launchObj,"Burnout.Geodetic.Longitude", "$Longitude deg");
    SetMCSSegmentValue(STKSocket1, $sat, $launchObj,"Burnout.FixedVelocity",      "$FixedVelocity km/sec");
	
	# --- Launch the satellite

	Send_Command_to_STK (STKSocket1, "Propagate $sat $start_time $stop_time");
	
	#--- Get Evaluation values
	
    $sma[$k]   = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.sma");
    $ecc[$k]   = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.ecc");
    $inc[$k]   = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.inc");
    $raan[$k]  = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.RAAN");
    $w[$k]     = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.w");
    $ta[$k]    = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.TA");
    $DV1       = GetMCSSegmentValue(STKSocket1, $sat, $mnvr1,   "FinalDeltaV");
    $DV2       = GetMCSSegmentValue(STKSocket1, $sat, $mnvr2,   "FinalDeltaV");
    $DVMag[$k] = $DV1 + $DV2;

}	

#--- Reset control parameters to initial values and get nominal values

print "\nResetting initial values...\n";

SetMCSSegmentValue(STKSocket1, $sat, $launchObj,"Burnout.Geodetic.Altitude",  "$initAltitude km");
SetMCSSegmentValue(STKSocket1, $sat, $launchObj,"Burnout.Geodetic.Latitude",  "$initLatitude deg");
SetMCSSegmentValue(STKSocket1, $sat, $launchObj,"Burnout.Geodetic.Longitude", "$initLongitude deg");
SetMCSSegmentValue(STKSocket1, $sat, $launchObj,"Burnout.FixedVelocity",      "$initFixedVelocity km/sec");

Send_Command_to_STK (STKSocket1, "Propagate $sat $start_time $stop_time");

$nom_sma   = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.sma");
$nom_ecc   = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.ecc");
$nom_inc   = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.inc");
$nom_raan  = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.RAAN");
$nom_w     = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.w");
$nom_ta    = GetMCSSegmentValue(STKSocket1, $sat, $evalObj, "FinalState.Keplerian.TA");
$nom_alat  = $nom_w + $nom_ta;
$DV1       = GetMCSSegmentValue(STKSocket1, $sat, $mnvr1,   "FinalDeltaV");
$DV2       = GetMCSSegmentValue(STKSocket1, $sat, $mnvr2,   "FinalDeltaV");
$nom_DVMag = $DV1 + $DV2;

#------------------------------
#--- Calculate statistics

#--- Calculate average

$tmp_sma   = 0;
$tmp_ecc   = 0;
$tmp_inc   = 0;
$tmp_raan  = 0;
$tmp_w     = 0;
$tmp_ta    = 0;
$tmp_alat  = 0;
$tmp_DVMag = 0;

for ($i=1; $i <= $numPoints; $i++)
{
	$tmp_sma   += $sma[$i];
	$tmp_ecc   += $ecc[$i];
	$tmp_inc   += $inc[$i];
	$tmp_raan  += $raan[$i];
	$tmp_w     += $w[$i];
	$tmp_ta    += $ta[$i];
	$tmp_alat  += $w[$i] + $ta[$i];
	$tmp_DVMag += $DVMag[$i];
}

$avg_sma   = $tmp_sma   / $numPoints;
$avg_ecc   = $tmp_ecc   / $numPoints;
$avg_inc   = $tmp_inc   / $numPoints;
$avg_raan  = $tmp_raan  / $numPoints;
$avg_w     = $tmp_w     / $numPoints;
$avg_ta    = $tmp_ta    / $numPoints;
$avg_alat  = $tmp_alat  / $numPoints;
$avg_DVMag = $tmp_DVMag / $numPoints;

#--- Calculate standard deviations

$tmp_sma   = 0;
$tmp_ecc   = 0;
$tmp_inc   = 0;
$tmp_raan  = 0;
$tmp_w     = 0;
$tmp_ta    = 0;
$tmp_alat  = 0;
$tmp_DVMag = 0;

for ($i=1; $i <= $numPoints; $i++)
{
	$diff = $sma[$i]         - $avg_sma;   $tmp_sma   += $diff * $diff;
	$diff = $ecc[$i]         - $avg_ecc;   $tmp_ecc   += $diff * $diff;
	$diff = $inc[$i]         - $avg_inc;   $tmp_inc   += $diff * $diff;
	$diff = $raan[$i]        - $avg_raan;  $tmp_raan  += $diff * $diff;
	$diff = $w[$i]           - $avg_w;     $tmp_w     += $diff * $diff;
	$diff = $ta[$i]          - $avg_ta;    $tmp_ta    += $diff * $diff;
	$diff = ($ta[$i]+$w[$i]) - $avg_alat;  $tmp_alat  += $diff * $diff;
	$diff = $DVMag[$i]       - $avg_DVMag; $tmp_DVMag += $diff * $diff;
}

$sig_sma   = sqrt( $tmp_sma  ) / ($numPoints - 1);
$sig_ecc   = sqrt( $tmp_ecc  ) / ($numPoints - 1);
$sig_inc   = sqrt( $tmp_inc  ) / ($numPoints - 1);
$sig_raan  = sqrt( $tmp_raan ) / ($numPoints - 1);
$sig_w     = sqrt( $tmp_w    ) / ($numPoints - 1);
$sig_ta    = sqrt( $tmp_ta   ) / ($numPoints - 1);
$sig_alat  = sqrt( $tmp_alat ) / ($numPoints - 1);
$sig_DVMag = sqrt( $tmp_DVMag) / ($numPoints - 1);


#--- Print results

print  "\n";
print  "\n*** Results using $numPoints trial runs";
print  "\n***";
print  "\n*** Value\tNominal\t\tAverage\t\tStdn Deviation";
print  "\n*** -----\t-------\t\t-------\t\t--------------";

printf "\n*** sma:  \t$form\t$form\t$form", $nom_sma,  $avg_sma,  $sig_sma;
printf "\n*** ecc:  \t$form\t$form\t%10.5g",$nom_ecc,  $avg_ecc,  $sig_ecc;
printf "\n*** inc:  \t$form\t$form\t$form", $nom_inc,  $avg_inc,  $sig_inc;
printf "\n*** raan: \t$form\t$form\t$form", $nom_raan, $avg_raan, $sig_raan;
printf "\n*** w:    \t$form\t$form\t$form", $nom_w,    $avg_w,    $sig_w;
printf "\n*** ta:   \t$form\t$form\t$form", $nom_ta,   $avg_ta,   $sig_ta;
printf "\n*** alat: \t$form\t$form\t$form", $nom_alat, $avg_alat, $sig_alat;
printf "\n*** DVMag:\t$form\t$form\t$form", $nom_DVMag,$avg_DVMag,$sig_DVMag;

print  "\n";

#--- Close

Close_Connection_to_STK(STKSocket1);

print "\n>>> Press the any key to continue.\n";
$pause=<STDIN>;

#----------------------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------------------
#----------------------------------------------------------------------------------------------------------------------

#--- Subroutines:

sub GetMCSSegmentValue
{
	local ($SocketName, $satellite, $object, $attribute) = @_;
	
    Send_Command_to_STK ($SocketName, "ASTROGATOR_RM $satellite GETVALUE MainSEQUENCE.SegmentList.$object.$attribute");
	
	@length = Read_STK_Output($SocketName);
	
	@returnString = Read_STK_Output($SocketName);
	
	($attr, $equal, $value, $unit) = split(/ /,$returnString[0]);
	
	return $value;

}

sub SetMCSSegmentValue
{
	local ($SocketName, $satellite, $object, $attribute, $value) = @_;
	
    Send_Command_to_STK (STKSocket1, "SETGATORVALUES $satellite MainSEQUENCE.SegmentList.$object.$attribute $value");
}

